@@ -1,8 +1,8 @@
-/* $Header: /usr/people/sam/tiff/libtiff/RCS/tif_dir.c,v 1.156 1996/01/10 20:37:08 sam Rel $ */
+/* $Header: /cvsroot/osrs/libtiff/libtiff/tif_dir.c,v 1.11 2000/03/03 15:15:57 warmerda Exp $ */
 
 /*
- * Copyright (c) 1988-1996 Sam Leffler
- * Copyright (c) 1991-1996 Silicon Graphics, Inc.
+ * Copyright (c) 1988-1997 Sam Leffler
+ * Copyright (c) 1991-1997 Silicon Graphics, Inc.
  *
  * Permission to use, copy, modify, distribute, and sell this software and 
  * its documentation for any purpose is hereby granted without fee, provided
@@ -50,6 +50,8 @@ _TIFFsetByteArray(void** vpp, void* vp, long n)
 }
 void _TIFFsetString(char** cpp, char* cp)
     { _TIFFsetByteArray((void**) cpp, (void*) cp, (long) (strlen(cp)+1)); }
+void _TIFFsetNString(char** cpp, char* cp, long n)
+    { _TIFFsetByteArray((void**) cpp, (void*) cp, n); }
 void _TIFFsetShortArray(uint16** wpp, uint16* wp, long n)
     { _TIFFsetByteArray((void**) wpp, (void*) wp, n*sizeof (uint16)); }
 void _TIFFsetLongArray(uint32** lpp, uint32* lp, long n)
@@ -82,6 +84,34 @@ setExtraSamples(TIFFDirectory* td, va_list ap, int* v)
 	return (1);
 }
 
+#ifdef CMYK_SUPPORT
+static int
+checkInkNamesString(TIFF* tif, int slen, const char* s)
+{
+	TIFFDirectory* td = &tif->tif_dir;
+	int i = td->td_samplesperpixel;
+
+	if (slen > 0) {
+		const char* ep = s+slen;
+		const char* cp = s;
+		for (; i > 0; i--) {
+			for (; *cp != '\0'; cp++)
+				if (cp >= ep)
+					goto bad;
+			cp++;				/* skip \0 */
+		}
+		return (cp-s);
+	}
+bad:
+	TIFFError("TIFFSetField",
+	    "%s: Invalid InkNames value; expecting %d names, found %d",
+	    tif->tif_name,
+	    td->td_samplesperpixel,
+	    td->td_samplesperpixel-i);
+	return (0);
+}
+#endif
+
 static int
 _TIFFVSetField(TIFF* tif, ttag_t tag, va_list ap)
 {
@@ -89,6 +119,8 @@ _TIFFVSetField(TIFF* tif, ttag_t tag, va_list ap)
 	int status = 1;
 	uint32 v32;
 	int i, v;
+	double d;
+	char* s;
 
 	switch (tag) {
 	case TIFFTAG_SUBFILETYPE:
@@ -135,8 +167,16 @@ _TIFFVSetField(TIFF* tif, ttag_t tag, va_list ap)
 		/*
 		 * Setup new compression routine state.
 		 */
-		if (status = TIFFSetCompressionScheme(tif, v))
-			td->td_compression = v;
+		if ( ! tif->tif_mode == O_RDONLY ) { 
+		  /* Handle removal of LZW compression */ 
+		  if ( v == COMPRESSION_LZW ) { 
+		    TIFFError(tif->tif_name, 
+			      "LZW compression no longer supported due to Unisys patent enforcement"); 
+		    v=COMPRESSION_NONE;
+		  }
+		}
+		if( (status = TIFFSetCompressionScheme(tif, v)) != 0 )
+		  td->td_compression = v;
 		break;
 	case TIFFTAG_PHOTOMETRIC:
 		td->td_photometric = (uint16) va_arg(ap, int);
@@ -312,6 +352,39 @@ _TIFFVSetField(TIFF* tif, ttag_t tag, va_list ap)
 	case TIFFTAG_IMAGEDEPTH:
 		td->td_imagedepth = va_arg(ap, uint32);
 		break;
+	case TIFFTAG_STONITS:
+		d = va_arg(ap, dblparam_t);
+		if (d <= 0.)
+			goto badvaluedbl;
+		td->td_stonits = d;
+		break;
+
+	/* Begin Pixar Tags */
+ 	case TIFFTAG_PIXAR_IMAGEFULLWIDTH:
+ 		td->td_imagefullwidth = va_arg(ap, uint32);
+ 		break;
+ 	case TIFFTAG_PIXAR_IMAGEFULLLENGTH:
+ 		td->td_imagefulllength = va_arg(ap, uint32);
+ 		break;
+ 	case TIFFTAG_PIXAR_TEXTUREFORMAT:
+ 		_TIFFsetString(&td->td_textureformat, va_arg(ap, char*));
+ 		break;
+ 	case TIFFTAG_PIXAR_WRAPMODES:
+ 		_TIFFsetString(&td->td_wrapmodes, va_arg(ap, char*));
+ 		break;
+ 	case TIFFTAG_PIXAR_FOVCOT:
+ 		td->td_fovcot = (float) va_arg(ap, dblparam_t);
+ 		break;
+ 	case TIFFTAG_PIXAR_MATRIX_WORLDTOSCREEN:
+ 		_TIFFsetFloatArray(&td->td_matrixWorldToScreen,
+ 			va_arg(ap, float*), 16);
+ 		break;
+ 	case TIFFTAG_PIXAR_MATRIX_WORLDTOCAMERA:
+ 		_TIFFsetFloatArray(&td->td_matrixWorldToCamera,
+ 			va_arg(ap, float*), 16);
+ 		break;
+ 	/* End Pixar Tags */	       
+
 #if SUBIFD_SUPPORT
 	case TIFFTAG_SUBIFD:
 		if ((tif->tif_flags & TIFF_INSUBIFD) == 0) {
@@ -364,11 +437,47 @@ _TIFFVSetField(TIFF* tif, ttag_t tag, va_list ap)
 		td->td_dotrange[1] = (uint16) va_arg(ap, int);
 		break;
 	case TIFFTAG_INKNAMES:
-		_TIFFsetString(&td->td_inknames, va_arg(ap, char*));
+		i = va_arg(ap, int);
+		s = va_arg(ap, char*);
+		i = checkInkNamesString(tif, i, s);
+                status = i > 0;
+		if( i > 0 ) {
+			_TIFFsetNString(&td->td_inknames, s, i);
+			td->td_inknameslen = i;
+		}
+		break;
+	case TIFFTAG_NUMBEROFINKS:
+		td->td_ninks = (uint16) va_arg(ap, int);
 		break;
 	case TIFFTAG_TARGETPRINTER:
 		_TIFFsetString(&td->td_targetprinter, va_arg(ap, char*));
 		break;
+#endif
+#ifdef ICC_SUPPORT
+	case TIFFTAG_ICCPROFILE:
+		td->td_profileLength = (uint32) va_arg(ap, uint32);
+		_TIFFsetByteArray(&td->td_profileData, va_arg(ap, void*),
+		    td->td_profileLength);
+		break;
+#endif
+#ifdef PHOTOSHOP_SUPPORT
+ 	case TIFFTAG_PHOTOSHOP:
+  		td->td_photoshopLength = (uint32) va_arg(ap, uint32);
+  		_TIFFsetByteArray (&td->td_photoshopData, va_arg(ap, void*),
+ 			td->td_photoshopLength);
+ 		break;
+#endif
+#ifdef IPTC_SUPPORT
+    case TIFFTAG_RICHTIFFIPTC: 
+  		td->td_richtiffiptcLength = (uint32) va_arg(ap, uint32);
+#ifdef PHOTOSHOP_SUPPORT
+  		_TIFFsetLongArray ((uint32**)&td->td_richtiffiptcData, va_arg(ap, uint32*),
+ 			td->td_richtiffiptcLength);
+#else
+  		_TIFFsetByteArray (&td->td_photoshopData, va_arg(ap, void*),
+ 			td->td_photoshopLength);
+#endif
+ 		break;
 #endif
 	default:
 		/*
@@ -405,6 +514,11 @@ badvalue32:
 	    _TIFFFieldWithTag(tif, tag)->field_name);
 	va_end(ap);
 	return (0);
+badvaluedbl:
+	TIFFError(tif->tif_name, "%f: Bad value for \"%s\"", d,
+	    _TIFFFieldWithTag(tif, tag)->field_name);
+	va_end(ap);
+	return (0);
 }
 
 /*
@@ -630,6 +744,9 @@ _TIFFVGetField(TIFF* tif, ttag_t tag, va_list ap)
 	case TIFFTAG_IMAGEDEPTH:
 		*va_arg(ap, uint32*) = td->td_imagedepth;
 		break;
+	case TIFFTAG_STONITS:
+		*va_arg(ap, double*) = td->td_stonits;
+		break;
 #if SUBIFD_SUPPORT
 	case TIFFTAG_SUBIFD:
 		*va_arg(ap, uint16*) = td->td_nsubifd;
@@ -677,10 +794,55 @@ _TIFFVGetField(TIFF* tif, ttag_t tag, va_list ap)
 	case TIFFTAG_INKNAMES:
 		*va_arg(ap, char**) = td->td_inknames;
 		break;
+	case TIFFTAG_NUMBEROFINKS:
+		*va_arg(ap, uint16*) = td->td_ninks;
+		break;
 	case TIFFTAG_TARGETPRINTER:
 		*va_arg(ap, char**) = td->td_targetprinter;
 		break;
 #endif
+#ifdef ICC_SUPPORT
+	case TIFFTAG_ICCPROFILE:
+		*va_arg(ap, uint32*) = td->td_profileLength;
+		*va_arg(ap, void**) = td->td_profileData;
+		break;
+#endif
+#ifdef PHOTOSHOP_SUPPORT
+ 	case TIFFTAG_PHOTOSHOP:
+ 		*va_arg(ap, uint32*) = td->td_photoshopLength;
+ 		*va_arg(ap, void**) = td->td_photoshopData;
+ 		break;
+#endif
+#ifdef IPTC_SUPPORT
+ 	case TIFFTAG_RICHTIFFIPTC:
+ 		*va_arg(ap, uint32*) = td->td_richtiffiptcLength;
+ 		*va_arg(ap, void**) = td->td_richtiffiptcData;
+ 		break;
+#endif
+ 	/* Begin Pixar Tags */
+ 	case TIFFTAG_PIXAR_IMAGEFULLWIDTH:
+ 		*va_arg(ap, uint32*) = td->td_imagefullwidth;
+ 		break;
+ 	case TIFFTAG_PIXAR_IMAGEFULLLENGTH:
+ 		*va_arg(ap, uint32*) = td->td_imagefulllength;
+ 		break;
+ 	case TIFFTAG_PIXAR_TEXTUREFORMAT:
+ 		*va_arg(ap, char**) = td->td_textureformat;
+ 		break;
+ 	case TIFFTAG_PIXAR_WRAPMODES:
+ 		*va_arg(ap, char**) = td->td_wrapmodes;
+ 		break;
+ 	case TIFFTAG_PIXAR_FOVCOT:
+ 		*va_arg(ap, float*) = td->td_fovcot;
+ 		break;
+ 	case TIFFTAG_PIXAR_MATRIX_WORLDTOSCREEN:
+ 		*va_arg(ap, float**) = td->td_matrixWorldToScreen;
+ 		break;
+ 	case TIFFTAG_PIXAR_MATRIX_WORLDTOCAMERA:
+ 		*va_arg(ap, float**) = td->td_matrixWorldToCamera;
+ 		break;
+ 	/* End Pixar Tags */
+
 	default:
 		/*
 		 * This can happen if multiple images are open with
@@ -774,9 +936,24 @@ TIFFFreeDirectory(TIFF* tif)
 	CleanupField(td_transferfunction[0]);
 	CleanupField(td_transferfunction[1]);
 	CleanupField(td_transferfunction[2]);
+#endif
+#ifdef ICC_SUPPORT
+	CleanupField(td_profileData);
+#endif
+#ifdef PHOTOSHOP_SUPPORT
+	CleanupField(td_photoshopData);
+#endif
+#ifdef IPTC_SUPPORT
+	CleanupField(td_richtiffiptcData);
 #endif
 	CleanupField(td_stripoffset);
 	CleanupField(td_stripbytecount);
+ 	/* Begin Pixar Tags */
+ 	CleanupField(td_textureformat);
+ 	CleanupField(td_wrapmodes);
+ 	CleanupField(td_matrixWorldToScreen);
+ 	CleanupField(td_matrixWorldToCamera);
+ 	/* End Pixar Tags */
 }
 #undef CleanupField
 
@@ -793,6 +970,26 @@ TIFFSetTagExtender(TIFFExtendProc extender)
 	return (prev);
 }
 
+/*
+ * Setup for a new directory.  Should we automatically call
+ * TIFFWriteDirectory() if the current one is dirty?
+ *
+ * The newly created directory will not exist on the file till
+ * TIFFWriteDirectory(), TIFFFlush() or TIFFClose() is called.
+ */
+int
+TIFFCreateDirectory(TIFF* tif)
+{
+    TIFFDefaultDirectory(tif);
+    tif->tif_diroff = 0;
+    tif->tif_nextdiroff = 0;
+    tif->tif_curoff = 0;
+    tif->tif_row = (uint32) -1;
+    tif->tif_curstrip = (tstrip_t) -1;
+
+    return 0;
+}
+
 /*
  * Setup a default directory structure.
  */
@@ -813,7 +1010,7 @@ TIFFDefaultDirectory(TIFF* tif)
 	td->td_tilelength = (uint32) -1;
 	td->td_tiledepth = 1;
 	td->td_resolutionunit = RESUNIT_INCH;
-	td->td_sampleformat = SAMPLEFORMAT_VOID;
+	td->td_sampleformat = SAMPLEFORMAT_UINT;
 	td->td_imagedepth = 1;
 #ifdef YCBCR_SUPPORT
 	td->td_ycbcrsubsampling[0] = 2;
@@ -822,6 +1019,7 @@ TIFFDefaultDirectory(TIFF* tif)
 #endif
 #ifdef CMYK_SUPPORT
 	td->td_inkset = INKSET_CMYK;
+	td->td_ninks = 4;
 #endif
 	tif->tif_postdecode = _TIFFNoPostDecode;
 	tif->tif_vsetfield = _TIFFVSetField;
@@ -849,31 +1047,73 @@ TIFFDefaultDirectory(TIFF* tif)
 static int
 TIFFAdvanceDirectory(TIFF* tif, uint32* nextdir, toff_t* off)
 {
-	static const char module[] = "TIFFAdvanceDirectory";
-	uint16 dircount;
+    static const char module[] = "TIFFAdvanceDirectory";
+    uint16 dircount;
+    if (isMapped(tif))
+    {
+        toff_t poff=*nextdir;
+        if (poff+sizeof(uint16) > tif->tif_size)
+        {
+            TIFFError(module, "%s: Error fetching directory count",
+                      tif->tif_name);
+            return (0);
+        }
+        _TIFFmemcpy(&dircount, tif->tif_base+poff, sizeof (uint16));
+        if (tif->tif_flags & TIFF_SWAB)
+            TIFFSwabShort(&dircount);
+        poff+=sizeof (uint16)+dircount*sizeof (TIFFDirEntry);
+        if (off != NULL)
+            *off = poff;
+        if (((toff_t) (poff+sizeof (uint32))) > tif->tif_size)
+        {
+            TIFFError(module, "%s: Error fetching directory link",
+                      tif->tif_name);
+            return (0);
+        }
+        _TIFFmemcpy(nextdir, tif->tif_base+poff, sizeof (uint32));
+        if (tif->tif_flags & TIFF_SWAB)
+            TIFFSwabLong(nextdir);
+        return (1);
+    }
+    else
+    {
+        if (!SeekOK(tif, *nextdir) ||
+            !ReadOK(tif, &dircount, sizeof (uint16))) {
+            TIFFError(module, "%s: Error fetching directory count",
+                      tif->tif_name);
+            return (0);
+        }
+        if (tif->tif_flags & TIFF_SWAB)
+            TIFFSwabShort(&dircount);
+        if (off != NULL)
+            *off = TIFFSeekFile(tif,
+                                dircount*sizeof (TIFFDirEntry), SEEK_CUR);
+        else
+            (void) TIFFSeekFile(tif,
+                                dircount*sizeof (TIFFDirEntry), SEEK_CUR);
+        if (!ReadOK(tif, nextdir, sizeof (uint32))) {
+            TIFFError(module, "%s: Error fetching directory link",
+                      tif->tif_name);
+            return (0);
+        }
+        if (tif->tif_flags & TIFF_SWAB)
+            TIFFSwabLong(nextdir);
+        return (1);
+    }
+}
 
-	if (!SeekOK(tif, *nextdir) ||
-	    !ReadOK(tif, &dircount, sizeof (uint16))) {
-		TIFFError(module, "%s: Error fetching directory count",
-		    tif->tif_name);
-		return (0);
-	}
-	if (tif->tif_flags & TIFF_SWAB)
-		TIFFSwabShort(&dircount);
-	if (off != NULL)
-		*off = TIFFSeekFile(tif,
-		    dircount*sizeof (TIFFDirEntry), SEEK_CUR);
-	else
-		(void) TIFFSeekFile(tif,
-		    dircount*sizeof (TIFFDirEntry), SEEK_CUR);
-	if (!ReadOK(tif, nextdir, sizeof (uint32))) {
-		TIFFError(module, "%s: Error fetching directory link",
-		    tif->tif_name);
-		return (0);
-	}
-	if (tif->tif_flags & TIFF_SWAB)
-		TIFFSwabLong(nextdir);
-	return (1);
+/*
+ * Count the number of directories in a file.
+ */
+tdir_t
+TIFFNumberOfDirectories(TIFF* tif)
+{
+    toff_t nextdir = tif->tif_header.tiff_diroff;
+    tdir_t n = 0;
+    
+    while (nextdir != 0 && TIFFAdvanceDirectory(tif, &nextdir, NULL))
+        n++;
+    return (n);
 }
 
 /*
@@ -883,7 +1123,7 @@ TIFFAdvanceDirectory(TIFF* tif, uint32* nextdir, toff_t* off)
 int
 TIFFSetDirectory(TIFF* tif, tdir_t dirn)
 {
-	uint32 nextdir;
+	toff_t nextdir;
 	tdir_t n;
 
 	nextdir = tif->tif_header.tiff_diroff;
@@ -939,7 +1179,7 @@ int
 TIFFUnlinkDirectory(TIFF* tif, tdir_t dirn)
 {
 	static const char module[] = "TIFFUnlinkDirectory";
-	uint32 nextdir;
+	toff_t nextdir;
 	toff_t off;
 	tdir_t n;
 
@@ -1003,3 +1243,53 @@ TIFFUnlinkDirectory(TIFF* tif, tdir_t dirn)
 	tif->tif_curstrip = (tstrip_t) -1;
 	return (1);
 }
+
+/*			[BFC]
+ *
+ * Author: Bruce Cameron <cameron@petris.com>
+ *
+ * Set a table of tags that are to be replaced during directory process by the
+ * 'IGNORE' state - or return TRUE/FALSE for the requested tag such that
+ * 'ReadDirectory' can use the stored information.
+ */
+int
+TIFFReassignTagToIgnore (enum TIFFIgnoreSense task, int TIFFtagID)
+{
+    static int TIFFignoretags [FIELD_LAST];
+    static int tagcount = 0 ;
+    int		i;					/* Loop index */
+    int		j;					/* Loop index */
+
+    switch (task)
+    {
+      case TIS_STORE:
+        if ( tagcount < (FIELD_LAST - 1) )
+        {
+            for ( j = 0 ; j < tagcount ; ++j )
+            {					/* Do not add duplicate tag */
+                if ( TIFFignoretags [j] == TIFFtagID )
+                    return (TRUE) ;
+            }
+            TIFFignoretags [tagcount++] = TIFFtagID ;
+            return (TRUE) ;
+        }
+        break ;
+        
+      case TIS_EXTRACT:
+        for ( i = 0 ; i < tagcount ; ++i )
+        {
+            if ( TIFFignoretags [i] == TIFFtagID )
+                return (TRUE) ;
+        }
+        break;
+        
+      case TIS_EMPTY:
+        tagcount = 0 ;			/* Clear the list */
+        return (TRUE) ;
+        
+      default:
+        break;
+    }
+    
+    return (FALSE);
+}
