@@ -1,8 +1,8 @@
-/* $Header: /usr/people/sam/tiff/libtiff/RCS/tif_dirread.c,v 1.71 1996/03/29 16:35:17 sam Rel $ */
+/* $Header: /cvsroot/osrs/libtiff/libtiff/tif_dirread.c,v 1.5 2000/01/28 21:00:20 warmerda Exp $ */
 
 /*
- * Copyright (c) 1988-1996 Sam Leffler
- * Copyright (c) 1991-1996 Silicon Graphics, Inc.
+ * Copyright (c) 1988-1997 Sam Leffler
+ * Copyright (c) 1991-1997 Silicon Graphics, Inc.
  *
  * Permission to use, copy, modify, distribute, and sell this software and 
  * its documentation for any purpose is hereby granted without fee, provided
@@ -59,9 +59,7 @@ static	int TIFFFetchFloatArray(TIFF*, TIFFDirEntry*, float*);
 static	int TIFFFetchDoubleArray(TIFF*, TIFFDirEntry*, double*);
 static	int TIFFFetchAnyArray(TIFF*, TIFFDirEntry*, double*);
 static	int TIFFFetchShortPair(TIFF*, TIFFDirEntry*);
-#if STRIPCHOP_SUPPORT
 static	void ChopUpSingleUncompressedStrip(TIFF*);
-#endif
 
 static char *
 CheckMalloc(TIFF* tif, tsize_t n, const char* what)
@@ -90,7 +88,7 @@ TIFFReadDirectory(TIFF* tif)
 	const TIFFFieldInfo* fip;
 	int fix;
 	uint16 dircount;
-	uint32 nextdiroff;
+	toff_t nextdiroff;
 	char* cp;
 	int diroutoforderwarning = 0;
 
@@ -100,8 +98,7 @@ TIFFReadDirectory(TIFF* tif)
 	/*
 	 * Cleanup any previous compression state.
 	 */
-	if (tif->tif_curdir != (tdir_t) -1)
-		(*tif->tif_cleanup)(tif);
+	(*tif->tif_cleanup)(tif);
 	tif->tif_curdir++;
 	nextdiroff = 0;
 	if (!isMapped(tif)) {
@@ -132,7 +129,7 @@ TIFFReadDirectory(TIFF* tif)
 	} else {
 		toff_t off = tif->tif_diroff;
 
-		if (off + sizeof (short) > tif->tif_size) {
+		if (off + sizeof (uint16) > tif->tif_size) {
 			TIFFError(tif->tif_name,
 			    "Can not read TIFF directory count");
 			return (0);
@@ -152,7 +149,7 @@ TIFFReadDirectory(TIFF* tif)
 			_TIFFmemcpy(dir, tif->tif_base + off,
 			    dircount*sizeof (TIFFDirEntry));
 		off += dircount* sizeof (TIFFDirEntry);
-		if (off + sizeof (uint32) < tif->tif_size)
+		if (off + sizeof (uint32) <= tif->tif_size)
 			_TIFFmemcpy(&nextdiroff, tif->tif_base+off, sizeof (uint32));
 	}
 	if (tif->tif_flags & TIFF_SWAB)
@@ -213,11 +210,17 @@ TIFFReadDirectory(TIFF* tif)
 	 */
 	fix = 0;
 	for (dp = dir, n = dircount; n > 0; n--, dp++) {
-		/*
-		 * Find the field information entry for this tag.
-		 */
+
+                /*
+                 * Find the field information entry for this tag.
+		 * Added check for tags to ignore ... [BFC]
+                 */
+		if( TIFFReassignTagToIgnore(TIS_EXTRACT, dp->tdir_tag) )
+                    dp->tdir_tag = IGNORE;
+
 		if (dp->tdir_tag == IGNORE)
-			continue;
+                    continue;
+                
 		/*
 		 * Silicon Beach (at least) writes unordered
 		 * directory tags (violating the spec).  Handle
@@ -528,7 +531,6 @@ TIFFReadDirectory(TIFF* tif)
 	 */
 	if (!TIFFFieldSet(tif, FIELD_COMPRESSION))
 		TIFFSetField(tif, TIFFTAG_COMPRESSION, COMPRESSION_NONE);
-#if STRIPCHOP_SUPPORT
         /*
          * Some manufacturers make life difficult by writing
 	 * large amounts of uncompressed data as a single strip.
@@ -538,11 +540,9 @@ TIFFReadDirectory(TIFF* tif)
 	 * side effect, however, is that the RowsPerStrip tag
 	 * value may be changed.
          */
-	if ((tif->tif_flags & TIFF_STRIPCHOP) &&
-	    td->td_nstrips == 1 && td->td_compression == COMPRESSION_NONE &&
-	    td->td_tilewidth == td->td_imagewidth)
+	if (td->td_nstrips == 1 && td->td_compression == COMPRESSION_NONE &&
+	    (tif->tif_flags & (TIFF_STRIPCHOP|TIFF_ISTILED)) == TIFF_STRIPCHOP)
 		ChopUpSingleUncompressedStrip(tif);
-#endif
 	/*
 	 * Reinitialize i/o since we are starting on a new directory.
 	 */
@@ -585,7 +585,9 @@ EstimateStripByteCounts(TIFF* tif, TIFFDirEntry* dir, uint16 dircount)
 			if (cc > sizeof (uint32))
 				space += cc;
 		}
-		space = (filesize - space) / td->td_samplesperpixel;
+		space = filesize - space;
+		if (td->td_planarconfig == PLANARCONFIG_SEPARATE)
+			space /= td->td_samplesperpixel;
 		for (i = 0; i < td->td_nstrips; i++)
 			td->td_stripbytecount[i] = space;
 		/*
@@ -596,12 +598,13 @@ EstimateStripByteCounts(TIFF* tif, TIFFDirEntry* dir, uint16 dircount)
 		 * of data in the strip and trim this number back accordingly.
 		 */ 
 		i--;
-		if (td->td_stripoffset[i] + td->td_stripbytecount[i] > filesize)
+		if (((toff_t)(td->td_stripoffset[i]+td->td_stripbytecount[i]))
+                                                               > filesize)
 			td->td_stripbytecount[i] =
 			    filesize - td->td_stripoffset[i];
 	} else {
 		uint32 rowbytes = TIFFScanlineSize(tif);
-		uint32 rowsperstrip = td->td_imagelength / td->td_nstrips;
+		uint32 rowsperstrip = td->td_imagelength/td->td_stripsperimage;
 		for (i = 0; i < td->td_nstrips; i++)
 			td->td_stripbytecount[i] = rowbytes*rowsperstrip;
 	}
@@ -753,28 +756,28 @@ TIFFFetchFloat(TIFF* tif, TIFFDirEntry* dir)
 static int
 TIFFFetchByteArray(TIFF* tif, TIFFDirEntry* dir, uint16* v)
 {
-	if (dir->tdir_count <= 4) {
-		/*
-		 * Extract data from offset field.
-		 */
-		if (tif->tif_header.tiff_magic == TIFF_BIGENDIAN) {
-			switch (dir->tdir_count) {
-			case 4: v[3] = dir->tdir_offset & 0xff;
-			case 3: v[2] = (dir->tdir_offset >> 8) & 0xff;
-			case 2: v[1] = (dir->tdir_offset >> 16) & 0xff;
-			case 1: v[0] = dir->tdir_offset >> 24;
-			}
-		} else {
-			switch (dir->tdir_count) {
-			case 4: v[3] = dir->tdir_offset >> 24;
-			case 3: v[2] = (dir->tdir_offset >> 16) & 0xff;
-			case 2: v[1] = (dir->tdir_offset >> 8) & 0xff;
-			case 1: v[0] = dir->tdir_offset & 0xff;
-			}
-		}
-		return (1);
-	} else
-		return (TIFFFetchData(tif, dir, (char*) v) != 0);	/* XXX */
+    if (dir->tdir_count <= 4) {
+        /*
+         * Extract data from offset field.
+         */
+        if (tif->tif_header.tiff_magic == TIFF_BIGENDIAN) {
+            switch (dir->tdir_count) {
+                case 4: v[3] = (uint16)(dir->tdir_offset & 0xff);
+                case 3: v[2] = (uint16)((dir->tdir_offset >> 8) & 0xff);
+                case 2: v[1] = (uint16)((dir->tdir_offset >> 16) & 0xff);
+                case 1: v[0] = (uint16)(dir->tdir_offset >> 24);
+            }
+        } else {
+            switch (dir->tdir_count) {
+                case 4: v[3] = (uint16)(dir->tdir_offset >> 24);
+                case 3: v[2] = (uint16)((dir->tdir_offset >> 16) & 0xff);
+                case 2: v[1] = (uint16)((dir->tdir_offset >> 8) & 0xff);
+                case 1: v[0] = (uint16)(dir->tdir_offset & 0xff);
+            }
+        }
+        return (1);
+    } else
+        return (TIFFFetchData(tif, dir, (char*) v) != 0);	/* XXX */
 }
 
 /*
@@ -786,13 +789,13 @@ TIFFFetchShortArray(TIFF* tif, TIFFDirEntry* dir, uint16* v)
 	if (dir->tdir_count <= 2) {
 		if (tif->tif_header.tiff_magic == TIFF_BIGENDIAN) {
 			switch (dir->tdir_count) {
-			case 2: v[1] = dir->tdir_offset & 0xffff;
-			case 1: v[0] = dir->tdir_offset >> 16;
+			case 2: v[1] = (uint16) (dir->tdir_offset & 0xffff);
+			case 1: v[0] = (uint16) (dir->tdir_offset >> 16);
 			}
 		} else {
 			switch (dir->tdir_count) {
-			case 2: v[1] = dir->tdir_offset >> 16;
-			case 1: v[0] = dir->tdir_offset & 0xffff;
+			case 2: v[1] = (uint16) (dir->tdir_offset >> 16);
+			case 1: v[0] = (uint16) (dir->tdir_offset & 0xffff);
 			}
 		}
 		return (1);
@@ -991,7 +994,7 @@ TIFFFetchAnyArray(TIFF* tif, TIFFDirEntry* dir, double* v)
 static int
 TIFFFetchNormalTag(TIFF* tif, TIFFDirEntry* dp)
 {
-	static char mesg[] = "to fetch tag value";
+	static const char mesg[] = "to fetch tag value";
 	int ok = 0;
 	const TIFFFieldInfo* fip = _TIFFFieldWithTag(tif, dp->tdir_tag);
 
@@ -1041,7 +1044,7 @@ TIFFFetchNormalTag(TIFF* tif, TIFFDirEntry* dp)
 			 * NULL byte, so always append one just in case.
 			 */
 			cp = CheckMalloc(tif, dp->tdir_count+1, mesg);
-			if (ok = (cp && TIFFFetchString(tif, dp, cp)))
+			if( (ok = (cp && TIFFFetchString(tif, dp, cp))) != 0 )
 				cp[dp->tdir_count] = '\0';	/* XXX */
 			break;
 		}
@@ -1114,7 +1117,7 @@ TIFFFetchNormalTag(TIFF* tif, TIFFDirEntry* dp)
 		case TIFF_ASCII:
 		case TIFF_UNDEFINED:		/* bit of a cheat... */
 			{ char c[2];
-			  if (ok = (TIFFFetchString(tif, dp, c) != 0)) {
+			  if( (ok = (TIFFFetchString(tif, dp, c) != 0)) != 0 ){
 				c[1] = '\0';		/* XXX paranoid */
 				ok = TIFFSetField(tif, dp->tdir_tag, c);
 			  }
@@ -1228,7 +1231,7 @@ TIFFFetchStripThing(TIFF* tif, TIFFDirEntry* dir, long nstrips, uint32** lpp)
 		    dir->tdir_count* sizeof (uint16), "to fetch strip tag");
 		if (dp == NULL)
 			return (0);
-		if (status = TIFFFetchShortArray(tif, dir, dp)) {
+		if( (status = TIFFFetchShortArray(tif, dir, dp)) != 0 ) {
 			register uint16* wp = dp;
 			while (nstrips-- > 0)
 				*lp++ = *wp++;
@@ -1271,7 +1274,7 @@ TIFFFetchExtraSamples(TIFF* tif, TIFFDirEntry* dir)
 static int
 TIFFFetchRefBlackWhite(TIFF* tif, TIFFDirEntry* dir)
 {
-	static char mesg[] = "for \"ReferenceBlackWhite\" array";
+	static const char mesg[] = "for \"ReferenceBlackWhite\" array";
 	char* cp;
 	int ok;
 
@@ -1281,10 +1284,10 @@ TIFFFetchRefBlackWhite(TIFF* tif, TIFFDirEntry* dir)
 	 * Handle LONG's for backward compatibility.
 	 */
 	cp = CheckMalloc(tif, dir->tdir_count * sizeof (uint32), mesg);
-	if (ok = (cp && TIFFFetchLongArray(tif, dir, (uint32*) cp))) {
+	if( (ok = (cp && TIFFFetchLongArray(tif, dir, (uint32*) cp))) != 0) {
 		float* fp = (float*)
 		    CheckMalloc(tif, dir->tdir_count * sizeof (float), mesg);
-		if (ok = (fp != NULL)) {
+		if( (ok = (fp != NULL)) != 0 ) {
 			uint32 i;
 			for (i = 0; i < dir->tdir_count; i++)
 				fp[i] = (float)((uint32*) cp)[i];
@@ -1298,7 +1301,6 @@ TIFFFetchRefBlackWhite(TIFF* tif, TIFFDirEntry* dir)
 }
 #endif
 
-#if STRIPCHOP_SUPPORT
 /*
  * Replace a single strip (tile) of uncompressed data by
  * multiple strips (tiles), each approximately 8Kbytes.
@@ -1353,7 +1355,7 @@ ChopUpSingleUncompressedStrip(TIFF* tif)
 	 * the broken-up format.
 	 */
 	for (strip = 0; strip < nstrips; strip++) {
-		if (stripbytes > bytecount)
+		if (stripbytes > (tsize_t) bytecount)
 			stripbytes = bytecount;
 		newcounts[strip] = stripbytes;
 		newoffsets[strip] = offset;
@@ -1371,4 +1373,3 @@ ChopUpSingleUncompressedStrip(TIFF* tif)
 	td->td_stripbytecount = newcounts;
 	td->td_stripoffset = newoffsets;
 }
-#endif /* STRIPCHOP_SUPPORT */
