@@ -1,8 +1,8 @@
-/* $Header: /usr/people/sam/tiff/libtiff/RCS/tif_win32.c,v 1.3 1996/01/10 19:33:18 sam Rel $ */
+/* $Header: /cvsroot/osrs/libtiff/libtiff/tif_win32.c,v 1.5 2000/01/28 21:42:48 warmerda Exp $ */
 
 /*
- * Copyright (c) 1988-1996 Sam Leffler
- * Copyright (c) 1991-1996 Silicon Graphics, Inc.
+ * Copyright (c) 1988-1997 Sam Leffler
+ * Copyright (c) 1991-1997 Silicon Graphics, Inc.
  *
  * Permission to use, copy, modify, distribute, and sell this software and 
  * its documentation for any purpose is hereby granted without fee, provided
@@ -52,23 +52,30 @@ _tiffWriteProc(thandle_t fd, tdata_t buf, tsize_t size)
 static toff_t
 _tiffSeekProc(thandle_t fd, toff_t off, int whence)
 {
-	DWORD dwMoveMethod;
+	DWORD dwMoveMethod, dwMoveHigh;
+
+        /* we use this as a special code, so avoid accepting it */
+        if( off == 0xFFFFFFFF )
+            return 0xFFFFFFFF;
+        
 	switch(whence)
 	{
-	case 0:
+	case SEEK_SET:
 		dwMoveMethod = FILE_BEGIN;
 		break;
-	case 1:
+	case SEEK_CUR:
 		dwMoveMethod = FILE_CURRENT;
 		break;
-	case 2:
+	case SEEK_END:
 		dwMoveMethod = FILE_END;
 		break;
 	default:
 		dwMoveMethod = FILE_BEGIN;
 		break;
 	}
-	return ((toff_t)SetFilePointer(fd, off, NULL, dwMoveMethod));
+        dwMoveHigh = 0;
+	return ((toff_t)SetFilePointer(fd, (LONG) off, (PLONG)&dwMoveHigh,
+                                       dwMoveMethod));
 }
 
 static int
@@ -83,60 +90,57 @@ _tiffSizeProc(thandle_t fd)
 	return ((toff_t)GetFileSize(fd, NULL));
 }
 
-/*
- * Because Windows uses both a handle and a pointer for file mapping, and only
- * the pointer is returned, the handle must be saved for later use (by the
- * unmap function).  To do this, the tiff structure has an extra member,
- * pv_map_handle, which is contiguous with (4 bytes or one 32-bit word above)
- * the tif_base parameter which is passed as *pbase to the map function.
- * pv_map_handle is then accessed indirectly (and perhaps somewhat unsafely)
- * as an offset from the *pbase parameter by _tiffMapProc.  The handle thus
- * created and saved is destroyed by _tiffUnmapProc, which does not need size
- * in Win32 but receives the map handle value in the size parameter instead.
- */
-
+#ifdef __BORLANDC__
 #pragma argsused
+#endif
 static int
 _tiffDummyMapProc(thandle_t fd, tdata_t* pbase, toff_t* psize)
 {
-	return(0);
+	return (0);
 }
 
+/*
+ * From "Hermann Josef Hill" <lhill@rhein-zeitung.de>:
+ *
+ * Windows uses both a handle and a pointer for file mapping,
+ * but according to the SDK documentation and Richter's book
+ * "Advanced Windows Programming" it is safe to free the handle
+ * after obtaining the file mapping pointer
+ *
+ * This removes a nasty OS dependency and cures a problem
+ * with Visual C++ 5.0
+ */
 static int
 _tiffMapProc(thandle_t fd, tdata_t* pbase, toff_t* psize)
 {
 	toff_t size;
-	HANDLE *phMapFile;
-	if ((size = _tiffSizeProc(fd)) == (toff_t)-1)
-		return(0);
-	phMapFile = (HANDLE *)(((BYTE *)pbase) + 4);
-	if ((*phMapFile = CreateFileMapping(fd, NULL, PAGE_READONLY, 0, size, NULL))
-			== NULL)
-		return(0);
-	if ((*pbase = MapViewOfFile(*phMapFile, FILE_MAP_READ, 0, 0, 0)) ==
-			NULL)
-	{
-		CloseHandle(*phMapFile);
-		*phMapFile = NULL;
-		return(0);
-	}
+	HANDLE hMapFile;
+
+	if ((size = _tiffSizeProc(fd)) == 0xFFFFFFFF)
+		return (0);
+	hMapFile = CreateFileMapping(fd, NULL, PAGE_READONLY, 0, size, NULL);
+	if (hMapFile == NULL)
+		return (0);
+	*pbase = MapViewOfFile(hMapFile, FILE_MAP_READ, 0, 0, 0);
+	CloseHandle(hMapFile);
+	if (*pbase == NULL)
+		return (0);
 	*psize = size;
 	return(1);
 }
 
+#ifdef __BORLANDC__
 #pragma argsused
+#endif
 static void
 _tiffDummyUnmapProc(thandle_t fd, tdata_t base, toff_t size)
 {
-	return;
 }
 
 static void
-_tiffUnmapProc(thandle_t fd, tdata_t base, toff_t map_handle)
+_tiffUnmapProc(thandle_t fd, tdata_t base, toff_t size)
 {
 	UnmapViewOfFile(base);
-	CloseHandle((HANDLE)map_handle);
-	return;
 }
 
 /*
@@ -183,7 +187,7 @@ TIFFOpen(const char* name, const char* mode)
 		dwMode = OPEN_ALWAYS;
 		break;
 	case O_RDWR|O_CREAT:
-		dwMode = CREATE_NEW;
+		dwMode = OPEN_ALWAYS;
 		break;
 	case O_RDWR|O_TRUNC:
 		dwMode = CREATE_ALWAYS;
@@ -223,7 +227,7 @@ _TIFFrealloc(tdata_t p, tsize_t s)
 	void* pvTmp;
 	if ((pvTmp = GlobalReAlloc(p, s, 0)) == NULL) {
 		if ((pvTmp = GlobalAlloc(GMEM_FIXED, s)) != NULL) {
-			CopyMemory(pvTmp, p, s);
+			CopyMemory(pvTmp, p, GlobalSize(p));
 			GlobalFree(p);
 		}
 	}
@@ -245,8 +249,8 @@ _TIFFmemcpy(void* d, const tdata_t s, tsize_t c)
 int
 _TIFFmemcmp(const tdata_t p1, const tdata_t p2, tsize_t c)
 {
-	register const BYTE *pb1 = p1;
-	register const BYTE *pb2 = p2;
+	register const BYTE *pb1 = (const BYTE *) p1;
+	register const BYTE *pb2 = (const BYTE *) p2;
 	register DWORD dwTmp = c;
 	register int iTmp;
 	for (iTmp = 0; dwTmp-- && !iTmp; iTmp = (int)*pb1++ - (int)*pb2++)
@@ -257,6 +261,7 @@ _TIFFmemcmp(const tdata_t p1, const tdata_t p2, tsize_t c)
 static void
 Win32WarningHandler(const char* module, const char* fmt, va_list ap)
 {
+#ifndef TIF_PLATFORM_CONSOLE
 	LPTSTR szTitle;
 	LPTSTR szTmp;
 	LPCTSTR szTitleText = "%s Warning";
@@ -271,12 +276,20 @@ Win32WarningHandler(const char* module, const char* fmt, va_list ap)
 	MessageBox(GetFocus(), szTmp, szTitle, MB_OK | MB_ICONINFORMATION);
 	LocalFree(szTitle);
 	return;
+#else
+	if (module != NULL)
+		fprintf(stderr, "%s: ", module);
+	fprintf(stderr, "Warning, ");
+	vfprintf(stderr, fmt, ap);
+	fprintf(stderr, ".\n");
+#endif        
 }
 TIFFErrorHandler _TIFFwarningHandler = Win32WarningHandler;
 
 static void
 Win32ErrorHandler(const char* module, const char* fmt, va_list ap)
 {
+#ifndef TIF_PLATFORM_CONSOLE
 	LPTSTR szTitle;
 	LPTSTR szTmp;
 	LPCTSTR szTitleText = "%s Error";
@@ -291,5 +304,11 @@ Win32ErrorHandler(const char* module, const char* fmt, va_list ap)
 	MessageBox(GetFocus(), szTmp, szTitle, MB_OK | MB_ICONEXCLAMATION);
 	LocalFree(szTitle);
 	return;
+#else
+	if (module != NULL)
+		fprintf(stderr, "%s: ", module);
+	vfprintf(stderr, fmt, ap);
+	fprintf(stderr, ".\n");
+#endif        
 }
 TIFFErrorHandler _TIFFerrorHandler = Win32ErrorHandler;
