@@ -1,4 +1,4 @@
-/* $Header: /cvsroot/osrs/libtiff/tools/rgb2ycbcr.c,v 1.2 1999/12/21 17:03:03 mwelles Exp $ */
+/* $Header: /cvsroot/osrs/libtiff/tools/rgb2ycbcr.c,v 1.3 1999/12/27 17:35:01 mwelles Exp $ */
 
 /*
  * Copyright (c) 1991-1997 Sam Leffler
@@ -321,7 +321,7 @@ static char* usageMsg[] = {
     "where comp is one of the following compression algorithms:\n",
     " jpeg\t\tJPEG encoding\n",
     " lzw\t\tLempel-Ziv & Welch encoding\n",
-    " (lzw no longer supported due to Unisys patent enforcement)", 
+    " (lzw no longer supported by default due to Unisys patent enforcement)", 
     " packbits\tPackBits encoding\n",
     " none\t\tno compression\n",
     "and the other options are:\n",
